<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
         xmlns:sql="http://symod.in2p3.fr/sql">
    <xsl:output method="text"/>
    <xsl:param name="md5">base64_md5_hash</xsl:param>
    <xsl:param name="reset-cache">false</xsl:param>

    <xsl:template match="sql:query">
        <xsl:param name="indent"><xsl:text>
</xsl:text></xsl:param>
        <xsl:if test="string-length($indent)>1"><xsl:value-of select="$indent"/></xsl:if>

        <xsl:if test="sql:select/sql:column/@limit">SELECT<xsl:text/>
            <xsl:for-each select="sql:select/sql:column[@limit]">
    CASE WHEN ROWNUM &lt;<xsl:text> </xsl:text><xsl:value-of select="@limit"/> THEN limited."<xsl:value-of select="@as"/>" ELSE 'Others' END AS "<xsl:value-of select="@as"/>",<xsl:text/>
            </xsl:for-each>
            <xsl:for-each select="sql:select/sql:column[not(@limit)]">
    SUM(limited."<xsl:value-of select="@as"/>") as "<xsl:value-of select="@as"/>"<xsl:text/>
                <xsl:if test="position() &lt; last()">,</xsl:if>
            </xsl:for-each>
FROM
(<xsl:value-of select="$indent"/>
        </xsl:if>

        <xsl:apply-templates select="sql:select"><xsl:with-param name="indent" select="$indent"/></xsl:apply-templates>
        <xsl:apply-templates select="sql:*[starts-with(local-name(),'from')]"><xsl:with-param name="indent" select="$indent"/></xsl:apply-templates>
        <xsl:apply-templates select="sql:where"><xsl:with-param name="indent" select="$indent"/></xsl:apply-templates>
        <xsl:apply-templates select="sql:group"><xsl:with-param name="indent" select="$indent"/></xsl:apply-templates>
        <xsl:apply-templates select="sql:having"><xsl:with-param name="indent" select="$indent"/></xsl:apply-templates>
        <xsl:apply-templates select="sql:order"><xsl:with-param name="indent" select="$indent"/></xsl:apply-templates>
        <xsl:apply-templates select="sql:pivot"><xsl:with-param name="indent" select="$indent"/></xsl:apply-templates>

        <xsl:if test="sql:select/sql:column/@limit">
) limited
GROUP BY<xsl:text/>
            <xsl:for-each select="sql:select/sql:column[@limit]">
    CASE WHEN ROWNUM &lt;<xsl:text> </xsl:text><xsl:value-of select="@limit"/> THEN limited."<xsl:value-of select="@as"/>" ELSE 'Others' END<xsl:text/>
            </xsl:for-each>
ORDER BY<xsl:text/>
            <xsl:for-each select="sql:select/sql:column[@limit]">
    CASE WHEN "<xsl:value-of select="@as"/>"='Others' THEN 1 ELSE 0 END,<xsl:text/>
            </xsl:for-each>
            <xsl:for-each select="sql:select/sql:column[not(@limit)]">
    "<xsl:value-of select="@as"/>" DESC<xsl:text/>
                <xsl:if test="position() &lt; last()">,</xsl:if>
            </xsl:for-each>
        </xsl:if>

        <xsl:if test="string-length($indent)=1"><xsl:value-of select="$indent"/></xsl:if>
    </xsl:template>
    <xsl:template match="sql:queryTimestampRef">
        <xsl:param name="indent"/>
        <xsl:variable name="period"><xsl:value-of select="@resolution"/>/1440<xsl:text/></xsl:variable>
        <xsl:value-of select="$indent"/>SELECT <xsl:text/>
        <xsl:call-template name="DATE">
            <xsl:with-param name="date">
                <xsl:value-of select="@since"/>
                <xsl:text/> + (<xsl:value-of select="/sql:query/@plot"/>/2-1)*<xsl:value-of select="$period"/>
                <xsl:if test="/sql:query/@singlePlot='true'"> + <xsl:value-of select="$period"/></xsl:if>
            </xsl:with-param>
        </xsl:call-template>
        <xsl:value-of select="$indent"/>    + (<xsl:value-of select="@resolution"/>/1440)*(rownum-1) AS "<xsl:value-of select="sql:field/@name"/>"<xsl:text/>
        <xsl:value-of select="$indent"/>FROM sys.dual<xsl:text/>
        <xsl:value-of select="$indent"/>CONNECT BY rownum &lt;= (((<xsl:value-of select="@until"/>) - (<xsl:value-of
            select="@since"/>)) * 1440/<xsl:value-of select="@resolution"/>)<xsl:text/>
        <xsl:if test="not(/sql:query/@singlePlot='true')"> + 1</xsl:if>
    </xsl:template>
    <xsl:template match="sql:queryUnion">
        <xsl:param name="indent"/>
        <xsl:for-each select="sql:*[starts-with(local-name(),'query')]">
            <xsl:if test="position() &gt; 1"><xsl:value-of select="$indent"/>UNION</xsl:if>
            <xsl:apply-templates select=".">
                <xsl:with-param name="indent" select="$indent"/>
            </xsl:apply-templates>
        </xsl:for-each>
    </xsl:template>
    <xsl:template match="sql:queryRef">
        <xsl:param name="indent"/>
        <xsl:variable name="queryRef" select="."/>
        <xsl:for-each select="/sql:query/sql:fromQueries/sql:query[@declarePrefix=$queryRef/@query]">
            <xsl:variable name="query" select="."/>
            <xsl:value-of select="$indent"/>SELECT<xsl:text/>
            <xsl:for-each select="$queryRef/sql:field">
                <xsl:variable name="fieldRef" select="."/>
                <xsl:apply-templates select="$query/sql:select/sql:column[
                                     sql:field/@prefix=$fieldRef/@prefix and sql:field/@name=$fieldRef/@name]">
                    <xsl:with-param name="indent" select="concat($indent, '    ')"/>
                </xsl:apply-templates>
            </xsl:for-each>
            <xsl:apply-templates select="sql:*[starts-with(local-name(),'from')]"><xsl:with-param name="indent" select="$indent"/></xsl:apply-templates>
            <xsl:apply-templates select="sql:where"><xsl:with-param name="indent" select="$indent"/></xsl:apply-templates>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="/sql:query[@cacheResolution or $reset-cache='true']/sql:order" priority="1"/>
    <xsl:template match="/sql:query[@cacheResolution or $reset-cache='true']/sql:select" priority="1">SELECT *</xsl:template>
    <xsl:template match="sql:select">
        <xsl:param name="indent"/>
        <xsl:text>SELECT</xsl:text>
        <xsl:if test="@distinct='true'"> DISTINCT</xsl:if>
        <xsl:choose>
            <xsl:when test="sql:column">
                <xsl:apply-templates select="sql:column">
                    <xsl:with-param name="indent" select="concat($indent, '    ')"/>
                </xsl:apply-templates>
            </xsl:when>
            <xsl:otherwise> *</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="sql:column">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/>
        <xsl:choose>
            <xsl:when test="@format">TO_CHAR(<xsl:apply-templates select="sql:field | sql:_"/>, '<xsl:value-of select="@format"/>')</xsl:when>
            <xsl:otherwise><xsl:apply-templates select="sql:field | sql:_"/></xsl:otherwise>
        </xsl:choose>
        <xsl:if test="@as"> AS "<xsl:value-of select="@as"/>"</xsl:if>
        <xsl:if test="position() &lt; last()">,</xsl:if>
    </xsl:template>

    <xsl:template match="sql:fromQueries">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/>FROM<xsl:text/>
        <xsl:choose>
            <xsl:when test="sql:*[starts-with(local-name(),'query')]">
                <xsl:if test="@relation='UNION ALL'">
                    <xsl:value-of select="$indent"/>(SELECT * FROM<xsl:text/>
                </xsl:if>
                <xsl:for-each select="sql:*[starts-with(local-name(),'query')]">
                    <xsl:if test="position()>1">
                        <xsl:value-of select="$indent"/><xsl:value-of select="parent::sql:fromQueries/@relation"/>
                    </xsl:if>
                    <xsl:value-of select="$indent"/>(<xsl:text/>
                    <xsl:apply-templates select=".">
                        <xsl:with-param name="indent" select="concat($indent, '    ')"/>
                    </xsl:apply-templates>
                    <xsl:value-of select="$indent"/>)<xsl:text/>
                    <xsl:if test="@declarePrefix"> "<xsl:value-of select="@declarePrefix"/>"</xsl:if>
                    <xsl:apply-templates select="sql:outerJoin">
                        <xsl:with-param name="indent" select="concat($indent, '    ')"/>
                    </xsl:apply-templates>
                </xsl:for-each>
                <xsl:if test="@relation='UNION ALL'">
                    <xsl:value-of select="$indent"/>) "<xsl:value-of select="sql:*[starts-with(local-name(),'query')]/@declarePrefix"/>"<xsl:text/>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise> DUAL</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="sql:from">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/>
        <xsl:choose>
            <xsl:when test="parent::sql:join">    INNER JOIN </xsl:when>
            <xsl:otherwise>FROM<xsl:value-of select="$indent"/><xsl:text>    </xsl:text></xsl:otherwise>
        </xsl:choose>
        <xsl:value-of select="/sql:query/@database"/>."<xsl:value-of select="@table"/>" "<xsl:value-of select="@as"/>"<xsl:text/>
        <xsl:apply-templates select="parent::sql:join">
            <xsl:with-param name="indent" select="concat($indent, '            ')"/>
        </xsl:apply-templates>
        <xsl:apply-templates select="sql:join/sql:from">
            <xsl:with-param name="indent" select="$indent"/>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="sql:join">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/>ON "<xsl:value-of select="parent::sql:from/@as"/>"."<xsl:value-of
            select="@parentField"/>" = "<xsl:value-of select="child::sql:from/@as"/>"."<xsl:value-of select="@childField"/>"<xsl:text/>
    </xsl:template>
    <xsl:template match="sql:outerJoin">
        <xsl:param name="indent"/>
        <xsl:text> </xsl:text>
        <xsl:choose>
            <xsl:when test="position() = 1">ON</xsl:when>
            <xsl:otherwise>AND</xsl:otherwise>
        </xsl:choose>
        <xsl:value-of select="$indent"/><xsl:apply-templates select="sql:field"/>="<xsl:value-of
            select="parent::sql:*/@declarePrefix"/>"."<xsl:value-of select="@on"/>"<xsl:text/>
    </xsl:template>

    <xsl:template match="sql:where[sql:and/sql:*]">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/>WHERE<xsl:text/>
        <xsl:value-of select="$indent"/>  (<xsl:text/>
        <xsl:apply-templates select="sql:and[sql:*]">
            <xsl:with-param name="indent" select="concat($indent, '    ')"/>
        </xsl:apply-templates>
        <xsl:value-of select="$indent"/>  )<xsl:text/>
    </xsl:template>
    <xsl:template match="sql:having[sql:and/sql:*]">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/>HAVING<xsl:text/>
        <xsl:value-of select="$indent"/>  (<xsl:text/>
        <xsl:apply-templates select="sql:and">
            <xsl:with-param name="indent" select="concat($indent, '    ')"/>
        </xsl:apply-templates>
        <xsl:value-of select="$indent"/>  )<xsl:text/>
    </xsl:template>
    <xsl:template match="sql:constraint">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/>
        <xsl:apply-templates select="sql:*[1]"/>
        <xsl:text> </xsl:text><xsl:value-of select="@operator"/>
        <xsl:text> </xsl:text><xsl:apply-templates select="sql:*[2]"/>
        <xsl:if test="position() &lt; last()"> AND</xsl:if>
    </xsl:template>
    <xsl:template match="sql:multi_constraint">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/><xsl:value-of select="text()"/>
        <xsl:if test="position() &lt; last()"> AND</xsl:if>
    </xsl:template>
    <xsl:template match="sql:and[sql:*]">
        <xsl:param name="indent"/>
        <xsl:variable name="i" select="substring($indent, 1, string-length($indent)-4)"/>
        <xsl:if test="position() &gt; 1">
            <xsl:value-of select="$i"/>  )<xsl:text/>
            <xsl:value-of select="$i"/>  AND<xsl:text/>
            <xsl:value-of select="$i"/>  (<xsl:text/>
        </xsl:if>
        <xsl:apply-templates select="sql:*">
            <xsl:with-param name="indent" select="$indent"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="sql:group[sql:by]">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/>GROUP BY<xsl:text/>
        <xsl:apply-templates select="sql:by">
            <xsl:with-param name="indent" select="concat($indent, '    ')"/>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="sql:group/sql:by">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/><xsl:apply-templates select="sql:*"/>
        <xsl:if test="position() &lt; last()">,</xsl:if>
    </xsl:template>
    <xsl:template match="sql:order[sql:by]">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/>ORDER BY<xsl:text/>
        <xsl:apply-templates select="sql:by">
            <xsl:with-param name="indent" select="concat($indent, '    ')"/>
        </xsl:apply-templates>
    </xsl:template>
    <xsl:template match="sql:order/sql:by">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/><xsl:apply-templates select="sql:*"/>
        <xsl:choose>
            <xsl:when test="@descending='true'"> DESC</xsl:when>
            <xsl:otherwise> ASC</xsl:otherwise>
        </xsl:choose>
        <xsl:if test="position() &lt; last()">,</xsl:if>
    </xsl:template>

    <xsl:template match="sql:pivot">
        <xsl:param name="indent"/>
        <xsl:value-of select="$indent"/>PIVOT XML<xsl:text/>
        <xsl:value-of select="$indent"/>(<xsl:text/>
        <xsl:value-of select="$indent"/><xsl:text>    </xsl:text>
        <xsl:for-each select="sql:column">
            <xsl:apply-templates select="."/>
            <xsl:if test="position() &lt; last()">, </xsl:if>
        </xsl:for-each>
        <xsl:value-of select="$indent"/>    FOR (<xsl:text/>
        <xsl:for-each select="sql:for">
            <xsl:text/>"<xsl:value-of select="@column"/>"<xsl:text/>
            <xsl:if test="position() &lt; last()">, </xsl:if>
        </xsl:for-each>
        <xsl:text>)</xsl:text>
        <xsl:value-of select="$indent"/>    IN (<xsl:text/>
        <xsl:for-each select="sql:for">
            <xsl:text/>ANY<xsl:text/>
            <xsl:if test="position() &lt; last()">, </xsl:if>
        </xsl:for-each>
        <xsl:text>)</xsl:text>
        <xsl:value-of select="$indent"/>)<xsl:text/>
    </xsl:template>

    <xsl:template match="sql:formula">
        <xsl:value-of select="text()"/>
    </xsl:template>
    <xsl:template match="sql:field">
        <xsl:value-of select="substring-before(text(), 'self')"/>
        <xsl:call-template name="REPLACE_SELF">
            <xsl:with-param name="expr">self<xsl:value-of select="substring-after(text(), 'self')"/></xsl:with-param>
        </xsl:call-template>
    </xsl:template>
    <xsl:template match="sql:constant">
        <xsl:call-template name="DATE">
            <xsl:with-param name="date">
                <xsl:value-of select="text()"/>
                <xsl:if test="@resolution">
                    <xsl:text/> + (<xsl:value-of select="/sql:query/@plot"/>/2<xsl:text/>
                    <xsl:if test="parent::sql:constraint/@operator='&lt;' or
                                  not(/sql:query/@singlePlot='true')">-1</xsl:if>
                    <xsl:text/>)*<xsl:value-of select="@resolution"/>/1440<xsl:text/>
                </xsl:if>
            </xsl:with-param>
        </xsl:call-template>
        <xsl:if test="@resolution">
            <xsl:text/> - (<xsl:value-of select="/sql:query/@plot"/>/2<xsl:text/>
            <xsl:if test="parent::sql:constraint/@operator='&lt;'">-1</xsl:if>
            <xsl:text/>)*<xsl:value-of select="@resolution"/>/1440<xsl:text/>
        </xsl:if>
    </xsl:template>
    <xsl:template match="sql:_">
        <xsl:value-of select="text()"/>
    </xsl:template>

    <xsl:template name="DATE">
        <xsl:param name="date"/>
        <xsl:choose>
            <xsl:when test="@resolution and /sql:query/@optimize='false'">
                <xsl:text/>DATE_ROUND(<xsl:value-of select="$date"/>, <xsl:value-of select="@resolution"/>)<xsl:text/>
            </xsl:when>
            <xsl:when test="@resolution">
                <xsl:text/>TO_DATE('1990_01_01','YYYY_MM_DD')+FLOOR(((<xsl:value-of
                    select="$date"/> - TO_DATE('1990_01_01','YYYY_MM_DD'))*1440)/<xsl:value-of
                    select="@resolution"/>)*<xsl:value-of select="@resolution"/>/1440<xsl:text/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$date"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="REPLACE_SELF">
        <xsl:param name="expr"/>
        <xsl:choose>
            <xsl:when test="contains($expr, 'self')">
                <xsl:value-of select="substring-before($expr, 'self')"/>
                <xsl:call-template name="DATE">
                    <xsl:with-param name="date">
                        <xsl:if test="@prefix">"<xsl:value-of select="@prefix"/>".</xsl:if>"<xsl:value-of select="@name"/>"<xsl:text/>
                        <xsl:if test="@resolution">
                            <xsl:text/> + (<xsl:value-of select="/sql:query/@plot"/>/2<xsl:text/>
                            <xsl:text/>)*<xsl:value-of select="@resolution"/>/1440<xsl:text/>
                        </xsl:if>
                    </xsl:with-param>
                </xsl:call-template>
                <xsl:call-template name="REPLACE_SELF">
                    <xsl:with-param name="expr" select="substring-after($expr, 'self')"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$expr"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="*"/>
</xsl:stylesheet>